///////////////////////////////////////////////////////////////////////////////////////////
// MAT 259 Assignment 4: Bird Migration Patterns Visualization
// Drawing Code (General) 
// by Nataly Moreno
//
// Links to download libraries:
//     Unfolded:  http://unfoldingmaps.org
//     ControlP5: http://www.sojamo.de/libraries/controlP5/#installation
//
///////////////////////////////////////////////////////////////////////////////////////////

final int DISPLAY_TIME = 1500;
final float MAX_ALPHA = 130.0;

float margin = 0.03;

//Fonts
PFont font1 = createFont("Arial", 22, true);     
PFont font2 = createFont("Helvetica", 27, true); 
PFont font3 = createFont("Arial Bold", 50, true);
PFont font4 = createFont("Arial", 12, true);
PFont font5 = createFont("Arial Bold", 24, true);



color blueColor     = color(8, 139, 225, 0);
color darkGrayColor = color(51, 50, 50, 202);
color darkBlueColor = color(50, 121, 162, 241);

String[] monthLabel = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
                        "Aug", "Sep", "Oct", "Nov", "Dec" };


//For the Framerate
int rate = 30;

//For Fading Animation
float alphi = 7.5 * MAX_ALPHA * (float)rate / DISPLAY_TIME;

int time = 0;
float alphaVal = MAX_ALPHA;
int lastTime;

//For Animation
String selectedBird = "";
float circleIncrease = 1.0;

int currentMonth = 0;
int current_week = 0;

//TimeLine Animation
float barLength = 396.0; //Has to be divisible by 4
float startTime = 0;
float step = (float)(barLength - ((float)barLength / 12.0)) / (float)(DISPLAY_TIME * 48.0);
float oldWidth = 0;

void draw() {
  background(240);
  map.draw();
    
  noFill();
  strokeWeight(1);
  stroke(modifiable);
        
  //Draw ControlP5 GUI
  drawGUI();
    
  //Draw Name of the State that is being Hovered Over
  if (currentMarker != null) 
  {
    colorMode(RGB);
    fill(255, 0, 0);
    textAlign(CENTER, CENTER);
    textFont(font5);
    text(currentMarker.getStringProperty("NAME"), width * 0.3, height * 0.15);
  }
  
  //Draw Time Line
  //drawTimeLine();
  drawTimeLineTitle();
  
  //Draw Color Bar
  drawColorBar(width * margin, height * 0.83);
  
  //////////////////////////////////////////////////////////////////////////////
  /////////////////////////// Draw Bird Count Points ///////////////////////////
  //////////////////////////////////////////////////////////////////////////////
  //Idea on how to code timed animations from: 
  //    http://forum.processing.org/one/topic/text-fade-in-out-over-specified-time.html
  //////////////////////////////////////////////////////////////////////////////

  if(selectedBird != "")
  {
    if(millis() - lastTime >= DISPLAY_TIME){
      alphaVal = MAX_ALPHA;  
      lastTime = millis(); 
      current_week++;
      circleIncrease = 1.0;
    
      if(current_week == 48)
      {
        current_week = 0;
        oldWidth  = 0;
        startTime = millis();
      }
    }
  
    //Title
    fill(darkGrayColor);
    textAlign(LEFT, BOTTOM);
    textFont(font3);
    //text("Bird Migration Pattern for: " + selectedBird, width * margin, height * 0.87);
    text("Bird Migration Patterns", width * margin, height * (1.0 - margin));

    //Print Month and Week
    /*fill(darkBlueColor);
    textAlign(CENTER, CENTER);
    textFont(font2);
    text(months[currentMonth] + ": Week " + ((current_week % 4) + 1), 
         width / 2, height * 0.07);
    */
    
    //TimeLine Ball Animation    
    /*float timePassed = (current_week - 1) * DISPLAY_TIME;
    timePassed = oldWidth + (millis() - lastTime);
    oldWidth = (step * (millis() - startTime));
    stroke(255, 0, 0, 200);
    fill(255, 0, 0, 200);
    
    pushMatrix();
    translate(5, 50);
    line((float)width * margin + oldWidth,
         (float)height * 0.8 - 15,
         (float)width * margin + oldWidth,
         (float)height * 0.8 + 15);
    popMatrix();
    */
    //ellipse((float)width * margin + oldWidth, 
    //        (float)height * 0.85, 15.0, 15.0);
    
    
    //Draw Bird's Photo
    drawImageFromGoogle();
    
    //Draw Circles
    noStroke();
      
    if(birdyData.get(selectedBird) != null)
    {
      Bird_Data tempAviData = birdyData.get(selectedBird);
    //for(int i = 0; i < 48; i++) //This is the data point
      for(int i = current_week; i < current_week + 1; i++) //This is the data point
      {
        chooseFillBasedOnTime((int) alphaVal, current_week);

        if(i % 4 == 0 && i < 48)
        {
          println("Month: " + months[(i / 4)]);
          currentMonth = (i / 4);
        }
        
      //println("    i = " + i);
      //println("birdStateSize: " + tempAviData.bird_states.size());
        for(int j = 0; j < tempAviData.bird_states.size(); j++) //This is the state 
        {
        //println("         - " + tempAviData.bird_states.get(j) + " " + 
        //  tempAviData.weekly_bird_data.get(j).get(i));
         // println("in for loop: " + i + ", " + j);
          Location l = stateCenters.get(tempAviData.bird_states.get(j));
          float yz[] = map.getScreenPositionFromLocation(l);
          ellipse(yz[0], yz[1], tempAviData.weekly_bird_data.get(j).get(i) * circleIncrease, 
                                tempAviData.weekly_bird_data.get(j).get(i) * circleIncrease);
        }
      //println("\n");
      }
    }
    else
    {
    //println("ERROR: Data for: " + selectedBird + " has disappeared!");
    }

    //alphaVal -= alphi;
    alphaVal -= (alphi + alphi * noise(current_week * millis()));
   
    if (alphaVal < 0 || alphaVal > MAX_ALPHA) { 
      alphi *= -1; 
    } 
    
    circleIncrease += 0.07;
  }
  else //No Bird is Selected
  {
    fill(darkGrayColor);
    textAlign(LEFT, CENTER);
    textFont(font3);
    text("Bird Migration Patterns", width * margin, height * (1.0 - margin));
  }
}

void drawTimeLineTitle()
{
  //Draw TimeLine Title
  if(selectedBird != "")
  {
    fill(darkGrayColor);
    textAlign(LEFT, BOTTOM);
    textFont(font2);
    text("Timeline: " + months[currentMonth] + ", Week " + ((current_week % 4) + 1), 
         width * margin, height * (1.0 - margin) - 60);
  }
}

void drawTimeLine()
{
  //Draw TimeLine Title
  if(selectedBird != "")
  {
    fill(darkGrayColor);
    textAlign(LEFT, BOTTOM);
    textFont(font2);
    text("Timeline: " + months[currentMonth] + ", Week " + ((current_week % 4) + 1), 
         width * margin, height * (1.0 - margin) - 60);
  }
  
  strokeWeight(4);
  stroke((blueColor & 0xffffff) | (255 << 24));
  pushMatrix();
  translate(5, 50);
  line(width * margin, height * 0.8, 
      (width * margin) + barLength, 
       height * 0.8);
  popMatrix();
  strokeWeight(2);
  
  //Draw Month Lines
  for(int i = 0; i < 13; i++)
  {
    pushMatrix();
    translate(5, 50);
    line(width * margin + (i * (barLength / 12)), height * 0.8 - 15,
         width * margin + (i * (barLength / 12)), height * 0.8 + 15);
    popMatrix();
    //Code Snippet to change alpha value:
    //    https://processing.org/discourse/beta/num_1261125421.html    
    fill((blueColor & 0xffffff) | (255 << 24));
    textAlign(CENTER, CENTER);
    textFont(font4);
    
    //if(i != 12)
    //  text(monthLabel[i], width * margin + (i * (barLength / 12)), height * 0.85 - 40);
  }
  
  //Draw Week Lines
  for(int i = 0; i < 48; i++)
  {
    pushMatrix();
    translate(5, 50);
    line(width * margin + (i * (barLength / 48)), height * 0.8 - 7,
         width * margin + (i * (barLength / 48)), height * 0.8 + 7);
    popMatrix();
    //Code Snippet to change alpha value:
    //    https://processing.org/discourse/beta/num_1261125421.html    
    fill((blueColor & 0xffffff) | (255 << 24));
    textAlign(CENTER, CENTER);
    textFont(font4);
    
    //if(i != 12)
    //  text(monthLabel[i], width * margin + (i * (barLength / 12)), height * 0.85 - 40);
  }
  
}









